@@ -14,7 +14,7 @@ Rcpp::List getMC(Eigen::VectorXd & beta, Eigen::VectorXd & tau,
                  const Eigen::MatrixXd & W, const Eigen::VectorXd & Y, 
                  const Eigen::MatrixXd & X2, const Eigen::VectorXd & survtime, 
                  const Eigen::VectorXd & cmprsk, const Eigen::VectorXd & mdata, 
-                 const Eigen::VectorXd & mdataS,
+                 const Eigen::VectorXi & mdataS,
                  const Eigen::VectorXd & FUNENW, 
                  const Eigen::MatrixXd & FUNBENW, 
                  const Eigen::MatrixXd & FUNBS, 
